// 40990 1.11 added "auto-close idle sockets"
// 1.22 changed the wait-state from defaults to "fast", resulting in the RD/WR pulses changing from ~750s to 3s each, changing the performance of AEW_DIO_ReadAll() from 5.5ms to 800s or so
// 1.23 improves the performance of WPDO by splitting it from RPDO
// 1.24 adds ReadAllWithLatch feature (NGC 2018-08-27)

// 1.27 2019-10-01 adds DHCP and Discovery features -- never released to public because Update (over the wire firmware upgrading) feature doesn't work
// 1.28 2019-10-07 fixes Update feature, which DHCP broke
// 1.29 2019-10-08 adding features

//#define DEBUG_PRINT

#ifndef h_eth_dio_48
#define h_eth_dio_48  

#define COMPANY "ACCES I/O Products, Inc."
#define MODEL "ETH-IIRO-16"

#define _ACTIVITY_LED_TIME_    0x80
#define _ACTIVITY_LED_ON_TIME_ 0x40

#ifndef DEBUG_PRINT
#define dprintf
#else
#define dprintf printf
#endif


#define VER_H		   (0x01)
#define VER_L		   (0x2A)

#define UPLD_PORT      51937
#define TALK_PORT      51936
#define DISC_PORT      51935
#define DISC_RESP_PORT 51934
#define DHCP_SOCK     0
#define DISC_UDP_SOCK 7 


typedef struct _StrConfigParam
{
	unsigned char op[4];		//0 ~ 3
	unsigned char ver[2];		//4 ~ 5
	unsigned char mac[6];		//6 ~ 11
	unsigned char ip[4];		//12 ~ 15
	unsigned char subnet[4];	//16 ~ 19
	unsigned char gw[4];		//20 ~ 23
	unsigned char dhcp;			//24
	unsigned char myMac[6]; 	//25 ~ 30	// the old mac address; Ver 1.3; 2010.02.08
	unsigned char pad;	
} StrConfigParam;

typedef struct _RawConfigParam
{
	unsigned char raw_data[32];
} RawConfigParam;

#define AIO_ETH_CMD_ChNP     0x43684E50
#define AIO_ETH_CMD_ChNW     0x43684E57
#define AIO_ETH_CMD_ChIP     0x43684950
#define AIO_ETH_CMD_ChIO     0x4368494F
#define AIO_ETH_CMD_ChGW     0x43684757
#define AIO_ETH_CMD_ChSM     0x4368534D
#define AIO_ETH_CMD_ChMC     0x43684D43
#define AIO_ETH_CMD_RSta     0x52537461

#define AIO_ETH_CMD_DIO_     0x44494F21
#define AIO_ETH_CMD_WADO     0x5741444F
#define AIO_ETH_CMD_RADI     0x52414449
#define AIO_ETH_CMD_RPDI     0x52504449
#define AIO_ETH_CMD_WPDO     0x5750444F
#define AIO_ETH_CMD_WRPD     0x57525044
#define AIO_ETH_CMD_RAWL     0x5241574C

#define AIO_ETH_CMD_Upld     0x55706c64
#define AIO_ETH_CMD_UDat     0x55446174
#define AIO_ETH_CMD_UFin     0x5546696E

#define AIO_ETH_CMD_WDBG     0x57444247


#define AIO_ETH_ACK_W_OK     0x575F4f4B
#define AIO_ETH_ACK_R_OK     0x525F4F4B
#define AIO_ETH_ACK__Err     0x5F457272


void ETH_IIRO_16_Go(SOCKET s, uint16 port, uint8  xdata * data_buf);
extern uint8      UPLOAD_STATUS;
extern bit        UPLOAD_MODE;
extern StrConfigParam NetworkParam;
extern uint8               Asynch_Reset;
extern uint8 fActivity;

typedef struct
{
    uint8     len;
    uint32    type;
    uint8     buf[200];
} t_dgram;

typedef struct
{
    uint8     len;
    uint32    type;
    uint32    linecount;
    uint16    filecsum;
} t_Upld_dgram;

typedef struct
{
    uint8     len;
    uint32    type;
    uint8     charcolon;
    uint16    linelength;
    uint32    lineaddr;
    uint16    linetype;
    uint8     buf[34];
} t_UDat_dgram;

sfr IOD = 0xB0;
sbit IOD_0 = 0xB0+0;
sbit IOD_1 = 0xB0+1;
sbit IOD_2 = 0xB0+2;
sbit IOD_3 = 0xB0+3;
sbit IOD_4 = 0xB0+4;
sbit IOD_5 = 0xB0+5;
sbit IOD_6 = 0xB0+6;
sbit IOD_7 = 0xB0+7;

sfr IOC = 0xA0;
sbit IOC_0 = 0xA0+0;
sbit IOC_1 = 0xA0+1;
sbit IOC_2 = 0xA0+2;
sbit IOC_3 = 0xA0+3;
sbit IOC_4 = 0xA0+4;
sbit IOC_5 = 0xA0+5;
sbit IOC_6 = 0xA0+6;
sbit IOC_7 = 0xA0+7;

#define XBYTE ((unsigned char volatile xdata *) 0)
#define x_data_firmware_storage_start (0x8000)

#define numdobytes 2
 
#define IDI07 (P1)
#define IDI8F (P3)

#define IDO07 (0)
#define IDO8F (1)

#define LED0 P2_7

#define OPT1_Jumper (IE0)
#define OPT2_Jumper (IE1)

#define WCONF_INIT_FOR_ADDR_BUS  0x28 // 0b00101000    // turn off ALE for bus access. See page 31 of wiznet W7100A datasheet
#define WCONF_INIT_DEFAULT       0x00 // 0b00000000    // power-on default value for this special-function-register


/* ethSimplePNPStruct work in process 2019-10-22

A preamble with hardware identifier(s), revision, serial number, etc.
followed by a pre-defined block of capabilities data
followed by zero or more key/value blocks, each highly structured yet flexible


typedef struct 
{
    unsigned char Length;
    unsigned char ID[2];
    unsigned char Rev[2];
    unsigned char SerialNumber[8]; // double   
} pnpPreamble;

typedef struct 
{
    unsigned char Length; 
    unsigned char iogroups;
    unsigned char diobytes;
    
} pnpCapabilities;
*/





#endif // header defined

















